import hashlib
import hmac


def generate_hmac(secret, base_string):
    """
    Helper method that returns an hmac from secret and base_string
    :param secret:
    :param base_string:
    :return:
    """
    if isinstance(secret, str):
        secret = secret.encode('UTF-8')

    if isinstance(base_string, str):
        base_string = base_string.encode('UTF-8')

    return hmac.new(secret, base_string, hashlib.sha256).hexdigest()


def verify_slack_request(request, slack_app_version, slack_app_signing_secret):
    """
    Helper method that verifies if the incoming request is from slack or not
    :param request:
    :param slack_app_version:
    :param slack_app_signing_secret:
    :return:
    """
    timestamp = request.headers['X-Slack-Request-Timestamp']
    body = request.body.decode('UTF-8')  # converting bytes to string
    base_string = f"{slack_app_version}:{timestamp}:{body}"

    generated_sign = f"{slack_app_version}={generate_hmac(slack_app_signing_secret, base_string)}"
    slack_sign = request.headers['X-Slack-Signature']

    return hmac.compare_digest(generated_sign, slack_sign)



from django.conf import settings

from slack_request_validator.abstract_permission import AbstractIsSlackAppRequest


class IsSlackAppRequest(AbstractIsSlackAppRequest):
    """
    Permission class to verify if the incoming request is coming from slack
    """
    SLACK_APP_VERSION = getattr(settings, 'SLACK_VERSION', 'v0')
    SLACK_APP_SIGNING_SECRET = getattr(settings, 'SLACK_APP_SIGNING_SECRET', '')



from rest_framework import permissions

from slack_request_validator.utils import verify_slack_request


class AbstractIsSlackAppRequest(permissions.BasePermission):
    """
    Abstract Class to be inherited if a new permission needs to be created for slack app request verification
    """

    class Meta:
        abstract = True

    MANDATORY_HEADERS = ('X-Slack-Request-Timestamp', 'X-Slack-Signature')
    SLACK_APP_VERSION = None
    SLACK_APP_SIGNING_SECRET = None

    def has_permission(self, request, view):
        if not all(header in request.headers for header in self.MANDATORY_HEADERS):
            return False

        return verify_slack_request(
            request=request,
            slack_app_version=self.SLACK_APP_VERSION,
            slack_app_signing_secret=self.SLACK_APP_SIGNING_SECRET
        )
